#include <SID.h>
#include <avr/pgmspace.h>
#include "music.h"

#define TICK 200

const uint16_t noteTable[] = { 
    16350, 17320, 18350, 19450, 20600, 
    21830, 23120, 24500, 25960, 27500, 
    29140, 30870 };

int musicPointer = -1;
unsigned long time;

void setup()
{
  sid.setFilterType(1,0,0);
  sid.setFilterMask(1,0,0);
  sid.setWaveform(0,2);
  sid.setPWM(0,1928);
  sid.setADSR(0,2,5,9,6);
  sid.setWaveform(1,8);
  sid.setADSR(1,0,8,4,7);
  sid.setWaveform(2,2);
  sid.setADSR(2,7,4,8,9);
  sid.setVolume(15);
  musicPointer = -1;
  time = millis();
}

void loop()
{
  int n;
  byte note;
  uint16_t freq;
  long elapsed;

  elapsed = millis()-time;
  time = millis();
  if (elapsed>0 && elapsed<TICK) {
    delay(TICK-elapsed);
  }

  for (n=0; n<3; n++) {
    note = pgm_read_byte_near(music + ++musicPointer);
    if (note==255) {
      musicPointer = -1;
      return;
    } else if (note==0) {
      sid.noteOff(n);
    } else {
      freq = note2freq(note);
      sid.noteOn(n,freq);
    }
  }

  freq = pgm_read_byte_near(music + ++musicPointer)<<3;
  sid.setCutoff(freq);
}

uint16_t note2freq(byte note)
{
  byte o = note/12;
  byte k = note-o*12;
  return (noteTable[k]>>(8-o));
}


